"use strict";

/**
 * For the onoff doc, 
 * see https://github.com/fivdi/onoff
 */

const State = {
  HIGH: 1,
  LOW: 0
};

const Direction = {
  IN: 'in',
  OUT: 'out'
};

const Event = {
  NONE: 'none',
  RISING: 'rising',
  FAILING: 'failing',
  BOTH: 'both'
};

let Gpio = require('onoff').Gpio; // Constructor function for Gpio objects.

function getMask(num) {
  let maskDim = 2;
  for (let i=2; i<16; i+=2) {
    maskDim = i;
    if (Math.abs(num) < (Math.pow(16, i) - 1)) {
//    console.log("i=" + i + ", " + Math.abs(num) + " < " + (Math.pow(16, i) - 1));
      break;
    }
  }
  return Math.pow(16, maskDim) - 1;
}

function lpad(str, len, pad) {
  let s = str;
  while (s.length < len) {
    s = (pad !== undefined ? pad : " ") + s;
  }
  return s;
}

function toHexString(num, len) {
  let l = (len !== undefined ? len : 4);
  return "0x" + lpad((num & getMask(num)).toString(16).trim().toUpperCase(), l, '0');
};

function toBinString(num, len) {
  let l = (len !== undefined ? len : 16);
  return "0&" + lpad((num & getMask(num)).toString(2).trim().toUpperCase(), l, '0');
};

Gpio.prototype.high = function() {
  this.writeSync(State.HIGH);
};    
Gpio.prototype.low = function() {
  this.writeSync(State.LOW);
};    
Gpio.prototype.isHigh = function() {
  return this.readSync() === State.HIGH;
};
Gpio.prototype.tick = function() {
  this.writeSync(State.HIGH);
  this.writeSync(State.LOW);
};

let defaultClock = 11; // GPIO_18, Wiring/PI4J 1
let defaultMiso  = 9; // GPIO_23, Wiring/PI4J 4
let defaultMosi  = 10; // GPIO_24, Wiring/PI4J 5
let defaultCs    = 5; // GPIO_25, Wiring/PI4J 6

var MCP3008 = function(clock, miso, mosi, cs) {

  if (clock === undefined) { clock = defaultClock; }
  if (miso === undefined)  { miso  = defaultMiso; }
  if (mosi === undefined)  { mosi = defaultMosi; }
  if (cs === undefined)    { cs = defaultCs; }

  let debug = false;
  if (debug)  console.log("Reading MCP3008: CLK:", clock, "MISO:", miso, "MOSI:", mosi, "CS:", cs);
  this.setDebug = function(val) {
    debug = val;
  };

  this.channels = {
    CHANNEL_0: 0,
    CHANNEL_1: 1,
    CHANNEL_2: 2,
    CHANNEL_3: 3,
    CHANNEL_4: 4,
    CHANNEL_5: 5,
    CHANNEL_6: 6,
    CHANNEL_7: 7 };

  let SPI_CLK  = new Gpio(clock, Direction.OUT),
      SPI_MISO = new Gpio(miso,  Direction.IN),
      SPI_MOSI = new Gpio(mosi,  Direction.OUT),
      SPI_CS   = new Gpio(cs,    Direction.OUT);

  SPI_CLK.low();
  SPI_MOSI.low();
  SPI_CS.low();

  this.readAdc = function(ch) {
    if (ch === undefined) {
      ch = this.channels.CHANNEL_0;
    }

    SPI_CS.high();
    SPI_CLK.low();
    SPI_CS.low();
    
    let adcCommand = ch;
    if (debug) {
      console.log(">> 1 -       ADCCOMMAND", toHexString(adcCommand, 4), toBinString(adcCommand, 16));
    }
    adcCommand |= 0x18; // 0x18: 00011000
    if (debug) {
      console.log(">> 2 -       ADCCOMMAND", toHexString(adcCommand, 4), toBinString(adcCommand, 16));
    }
    adcCommand <<= 3;
    if (debug) {
      console.log(">> 3 -       ADCCOMMAND", toHexString(adcCommand, 4), toBinString(adcCommand, 16));
    }
    // Send 5 bits: 8 - 3. 8 input channels on the MCP3008.
    for (let i=0; i<5; i++) {
      if (debug) {
        console.log(">> 4 - (i=" + i + ") ADCCOMMAND", toHexString(adcCommand, 4), toBinString(adcCommand, 16));
      }
      if ((adcCommand & 0x80) != 0x0) { // 0x80 = 0&10000000
        SPI_MOSI.high();
      } else {
        SPI_MOSI.low();
      }
      adcCommand <<= 1;
      SPI_CLK.tick();
    }
	  let adcOut = 0;
    for (let i=0; i<12; i++) { // Read in one empty bit, one null bit and 10 ADC bits
      SPI_CLK.tick();
      adcOut <<= 1;

      if (SPI_MISO.isHigh()) {
        adcOut |= 0x1;
      }
      if (debug) {
        console.log(">> ADCOUT", toHexString(adcOut, 4), toBinString(adcOut, 16));
      }
    }
    SPI_CS.high();
    adcOut >>= 1; // Drop first bit
    return adcOut;
  };

  this.shutdown = function() {
    SPI_CLK.unexport();
    SPI_MISO.unexport();
    SPI_CS.unexport();
    SPI_MOSI.unexport();
  };
};

exports.MCP3008 = MCP3008; // Made public.

